"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tracing = exports.metrics = void 0;
const dummy = { hasSubscribers: false };
exports.metrics = dummy;
exports.tracing = dummy;
//# sourceMappingURL=diagnostics-channel-cjs.cjs.map