/**
 * this provides the default Perf object source, either the
 * `performance` global, or the `Date` constructor.
 *
 * it can be passed in via configuration to override it
 * for a single LRU object.
 */
export type Perf = {
    now: () => number;
};
export declare const defaultPerf: Perf;
//# sourceMappingURL=perf.d.ts.map